obs         = obslua
bit = require("bit")
source_name = ""
hotkey_id   = obs.OBS_INVALID_HOTKEY_ID

----------------------------------------------------------

-- The "Rotate Source 180°" hotkey callback
function rotate_source(pressed)
	if not pressed then
		return
	end

	local found = true
	local scene_source = obs.obs_frontend_get_current_scene()
	if scene_source ~= nil then
		local scene = obs.obs_scene_from_source(scene_source)
		local item = obs.obs_scene_find_source(scene, source_name)
		if item ~= nil then
			found = true

			local info = obs.obs_transform_info()
			obs.obs_sceneitem_get_info(item, info)
			local source = obs.obs_sceneitem_get_source(item)
			local width = obs.obs_source_get_width(source) * info.scale.x
			local height = obs.obs_source_get_height(source) * info.scale.y

			function get_tl(item)
				local info = obs.obs_transform_info()
				obs.obs_sceneitem_get_info(item, info)
				local source = obs.obs_sceneitem_get_source(item)
				local width = obs.obs_source_get_width(source) * info.scale.x
				local height = obs.obs_source_get_height(source) * info.scale.y

				local OBS_ALIGN_CENTER = 0
				local OBS_ALIGN_LEFT = 1
				local OBS_ALIGN_RIGHT = 2
				local OBS_ALIGN_TOP = 4
				local OBS_ALIGN_BOTTOM = 8

				local tl = obs.vec2()
				local br = obs.vec2()
				if info.alignment == OBS_ALIGN_CENTER then
					obs.vec2_set( tl, -width / 2, -height / 2 )
					obs.vec2_set( br, width / 2, height / 2 )
				else
					if bit.band(info.alignment, OBS_ALIGN_LEFT) ~= 0 then
						tl.x = 0.0
						br.x = width
					elseif bit.band(info.alignment, OBS_ALIGN_RIGHT) ~= 0 then
						tl.x = -width
						br.x = 0.0
					else
						tl.x = -width / 2
						br.x = width / 2
					end
					if bit.band(info.alignment, OBS_ALIGN_TOP) ~= 0 then
						tl.y = 0.0
						br.y = height
					elseif bit.band(info.alignment, OBS_ALIGN_BOTTOM) ~= 0 then
						tl.y = -height
						br.y = 0.0
					else
						tl.y = -height / 2
						br.y = height / 2
					end
				end

				local tr = obs.vec2()
				obs.vec2_set(tr, br.x, tl.y)

				local bl = obs.vec2()
				obs.vec2_set(bl, tl.x, br.y)

				if info.rot ~= 0.0 then
					local rot_rad = info.rot * math.pi / 180.0
					local c = math.cos(rot_rad)
					local s = math.sin(rot_rad)
					obs.vec2_set(tl, tl.x * c - tl.y * s, tl.x * s + tl.y * c)
					obs.vec2_set(tr, tr.x * c - tr.y * s, tr.x * s + tr.y * c)
					obs.vec2_set(bl, bl.x * c - bl.y * s, bl.x * s + bl.y * c)
					obs.vec2_set(br, br.x * c - br.y * s, br.x * s + br.y * c)
				end

				tl.x = tl.x + info.pos.x
				tl.y = tl.y + info.pos.y
				tr.x = tr.x + info.pos.x
				tr.y = tr.y + info.pos.y
				bl.x = bl.x + info.pos.x
				bl.y = bl.y + info.pos.y
				br.x = br.x + info.pos.x
				br.y = br.y + info.pos.y

				if tr.x < tl.x then tl.x = tr.x end
				if bl.x < tl.x then tl.x = bl.x end
				if br.x < tl.x then tl.x = br.x end
				if tr.y < tl.y then tl.y = tr.y end
				if bl.y < tl.y then tl.y = bl.y end
				if br.y < tl.y then tl.y = br.y end

				return tl
			end

			local tl = get_tl(item)
			
			local rot = obs.obs_sceneitem_get_rot(item)
			rot = rot + 180.0
			if rot >= 360.0 then
				rot = rot - 360.0
			elseif rot <= -360.0 then
				rot = rot + 360.0
			end
			obs.obs_sceneitem_set_rot(item, rot)

			local new_tl = get_tl(item)

			local pos = obs.vec2()
			obs.obs_sceneitem_get_pos(item, pos)
			pos.x = pos.x + tl.x - new_tl.x
			pos.y = pos.y + tl.y - new_tl.y
			obs.obs_sceneitem_set_pos(item, pos)

		end
		obs.obs_source_release(scene_source)
	end

	if not found then
		obs.script_log(obs.LOG_WARNING, "Source " .. source_name .. " not found.")
	end
end

----------------------------------------------------------

-- A function named script_update will be called when settings are changed
function script_update(settings)
	source_name = obs.obs_data_get_string(settings, "source")
end

-- A function named script_description returns the description shown to
-- the user
function script_description()
	return "When the \"Rotate Source 180°\" hotkey is triggered, the specified source will be rotated 180 degress."
end

-- A function named script_properties defines the properties that the user
-- can change for the entire script module itself
function script_properties()
	props = obs.obs_properties_create()

	local p = obs.obs_properties_add_list(props, "source", "Source", obs.OBS_COMBO_TYPE_EDITABLE, obs.OBS_COMBO_FORMAT_STRING)
	local sources = obs.obs_enum_sources()
	if sources ~= nil then
		for _, source in ipairs(sources) do
			source_id = obs.obs_source_get_id(source)
			local name = obs.obs_source_get_name(source)
			obs.obs_property_list_add_string(p, name, name)
		end
	end
	obs.source_list_release(sources)

	return props
end

-- A function named script_load will be called on startup
function script_load(settings)
	hotkey_id = obs.obs_hotkey_register_frontend("rotate_source.trigger", "Rotate Source 180°", rotate_source)
	local hotkey_save_array = obs.obs_data_get_array(settings, "rotate_source.trigger")
	obs.obs_hotkey_load(hotkey_id, hotkey_save_array)
	obs.obs_data_array_release(hotkey_save_array)
end

-- A function named script_save will be called when the script is saved
--
-- NOTE: This function is usually used for saving extra data (such as in this
-- case, a hotkey's save data).  Settings set via the properties are saved
-- automatically.
function script_save(settings)
	local hotkey_save_array = obs.obs_hotkey_save(hotkey_id)
	obs.obs_data_set_array(settings, "rotate_source.trigger", hotkey_save_array)
	obs.obs_data_array_release(hotkey_save_array)
end
